From 16c8f82ba88ac17f1735b1b56d844d437c2e7872 Mon Sep 17 00:00:00 2001
From: Smeulf <Smeulf@media-portal.com>
Date: Fri, 25 Feb 2011 22:30:16 +0100
Subject: [PATCH] Fix AudioRingBuffer Read() and Peek() methods to avoid scratches

---
 .../UI/Players/BassPlayer/Utils/AudioRingBuffer.cs |   51 +++++++++++---------
 1 files changed, 28 insertions(+), 23 deletions(-)

diff --git a/MediaPortal/Source/UI/Players/BassPlayer/Utils/AudioRingBuffer.cs b/MediaPortal/Source/UI/Players/BassPlayer/Utils/AudioRingBuffer.cs
index aa33b47..a3bf968 100644
--- a/MediaPortal/Source/UI/Players/BassPlayer/Utils/AudioRingBuffer.cs
+++ b/MediaPortal/Source/UI/Players/BassPlayer/Utils/AudioRingBuffer.cs
@@ -219,19 +219,23 @@ namespace Ui.Players.BassPlayer.Utils
       else
       {
         int count1 = Math.Min(requested, _bufferLength - readPointer);
-
-        Marshal.Copy(_buffer, readPointer, buffer, count1);
-        readPointer += count1;
-
-        if (readPointer == _buffer.Length)
-          readPointer = 0;
-
         int count2 = Math.Min(requested - count1, writePointer);
-        if (count2 > 0)
+        if (count2 <= 0)
         {
-          IntPtr ptr = new IntPtr(_Is32bit ? buffer.ToInt32() : buffer.ToInt64() + (count1 * BassConstants.FloatBytes));
+          if (buffer != IntPtr.Zero)
+            Marshal.Copy(_buffer, readPointer, buffer, count1);
+
+          readPointer += count1;
 
-          Marshal.Copy(_buffer, 0, ptr, count2);
+          if (readPointer == _buffer.Length)
+            readPointer = 0;
+        }
+        else
+        {
+          float[] tempArray = new float[requested];
+          Array.Copy(_buffer, readPointer, tempArray, 0, count1);
+          Array.Copy(_buffer, 0, tempArray, count1, count2);
+          Marshal.Copy(tempArray, 0, buffer, count1 + count2);
           readPointer = count2;
         }
         read = count1 + count2;
@@ -281,22 +285,23 @@ namespace Ui.Players.BassPlayer.Utils
       else
       {
         int count1 = Math.Min(requested, _bufferLength - readPointer);
-
-        if (buffer != IntPtr.Zero)
-          Marshal.Copy(_buffer, readPointer, buffer, count1);
-        readPointer += count1;
-
-        if (readPointer == _buffer.Length)
-          readPointer = 0;
-
         int count2 = Math.Min(requested - count1, writePointer);
-        if (count2 > 0)
+        if (count2 <= 0)
         {
           if (buffer != IntPtr.Zero)
-          {
-            IntPtr ptr = new IntPtr(_Is32bit ? buffer.ToInt32() : buffer.ToInt64() + (count1 * BassConstants.FloatBytes));
-            Marshal.Copy(_buffer, 0, ptr, count2);
-          }
+            Marshal.Copy(_buffer, readPointer, buffer, count1);
+
+          readPointer += count1;
+
+          if (readPointer == _buffer.Length)
+            readPointer = 0;
+        }
+        else
+        {
+          float[] tempArray = new float[requested];
+          Array.Copy(_buffer, readPointer, tempArray, 0, count1);
+          Array.Copy(_buffer, 0, tempArray, count1, count2);
+          Marshal.Copy(tempArray, 0, buffer, count1 + count2);
           readPointer = count2;
         }
         read = count1 + count2;
-- 
1.7.3.1.msysgit.0

